/*
 * Copyright (C) 2000 Robert Fitzsimons
 */

#ifndef _ujvm_classfile_h
#define _ujvm_classfile_h

#include "basetypes.h"
#include "datatypes.h"

struct ClassFile {
	kuint magic;
	kuint minor_version;
	kuint major_version;
	kuint constant_count;
	ConstantInfo* constant;
	kuint access_flags;
	kuint this_class;
	kuint super_class;
	kuint interface_count;
	kuint* interface;
	kuint field_count;
	FieldInfo* field;
	kuint method_count;
	MethodInfo* method;
	kuint attribute_count;
	AttributeInfo* attribute;
};

union ConstantInfo {
	struct {
		kuint tag;
	} info;
	struct {
		kuint tag;
		kuint length;
		kuint8* bytes;
	} utf8_info;
	struct {
		kuint tag;
		kuint32 bytes;
	} v32_info;
	struct {
		kuint tag;
		kuint32 high_bytes;
		kuint32 low_bytes;
	} v64_info;
	struct {
		kuint tag;
		kuint string_index;
	} string_info;
	struct {
		kuint tag;
		kuint name_index;
	} class_info;
	struct {
		kuint tag;
		kuint class_index;
		kuint nametype_index;
	} ref_info;
	struct {
		kuint tag;
		kuint name_index;
		kuint descriptor_index;
	} nametype_info;
};

struct FieldInfo {
	kuint access_flags;
	kuint name_index;
	kuint descriptor_index;
	kuint attribute_count;
	AttributeInfo* attribute;
};

struct MethodInfo {
	kuint access_flags;
	kuint name_index;
	kuint descriptor_index;
	kuint attribute_count;
	AttributeInfo* attribute;
};

struct AttributeInfo {
	kuint name_index;
	kuint length;
	kuint8* array;
};

#define CONSTANT_Info_Utf8 0x01
#define CONSTANT_Info_Integer 0x03
#define CONSTANT_Info_Float 0x04
#define CONSTANT_Info_Long 0x05
#define CONSTANT_Info_Double 0x06
#define CONSTANT_Info_Class 0x07
#define CONSTANT_Info_String 0x08
#define CONSTANT_Info_FieldRef 0x09
#define CONSTANT_Info_MethodRef 0x0A
#define CONSTANT_Info_InterfaceMethodRef 0x0B
#define CONSTANT_Info_NameAndType 0x0C

ClassFile* classfile_build(kuint8* buffer, kuint buffer_length);
void classfile_free(ClassFile* classfile);

void classfile_print(ClassFile* classfile);

kuint read_u1(kuint8** buffer_pointer);
kuint read_u2(kuint8** buffer_pointer);
kuint read_u4(kuint8** buffer_pointer);

#endif /* _ujvm_classfile_h */

